#!/usr/bin/env python
# -*- coding: utf-8 -*-
# PyBEST: Pythonic Black-box Electronic Structure Tool
# Copyright (C) 2016-- The PyBEST Development Team
#
# This file is part of PyBEST.
#
# PyBEST is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# PyBEST is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --


import numpy as np, argparse
import pathlib
import matplotlib.image as mpimg
import matplotlib.colors as colors
import matplotlib.cm as cm
from matplotlib.backends.backend_pdf import PdfPages

from pybest import __version__
from pybest import filemanager
from pybest.log import log
from pybest.exceptions import ArgumentError


def ploti12(
    i12ind1,
    i12ind2,
    i12val,
    orbinit,
    orbfinal,
    thresh,
    s1index,
    s1value,
    indices,
    smax=0.71,
    zoom=0.09,
):
    try:
        import matplotlib.pyplot as plt
        import matplotlib.lines as mlines
        from matplotlib.ticker import NullFormatter
    except ImportError:
        if log.do_warning:
            log.warn("Skipping plots because matplotlib was not found.")
        return

    plt.rcParams.update(
        {
            "text.usetex": 1,
            # line below turns the ticks to bold
            "text.latex.preamble": [r"\boldmath"],
        }
    )

    norb = orbfinal - orbinit + 1
    orbitals = np.arange(orbinit, orbfinal + 1)
    theta = 2 * np.pi * (orbitals - orbinit) / (norb)
    MAX_R = 150.0
    radius = np.ones(norb, int) * (MAX_R + 50.0)
    r = (MAX_R) * np.ones(norb, int) - 15.00 * ((orbitals - orbinit) % 2)

    plt.figure(figsize=(10, 10))
    #
    # First subplot: mutual information
    #
    ax = plt.subplot(111, polar=True)
    ax.grid(False)
    ax.set_theta_zero_location("N")
    ax.set_theta_direction(-1)
    ax.spines["polar"].set_visible(False)

    for i in range(len(orbitals)):
        offset = -10
        if i % 2 == 0:
            offset = 10
        ax.annotate(
            indices[i],
            # i+1+orbinit,
            xy=(theta[i], r[i] + offset),
            xytext=(0, 0),
            textcoords="offset points",  # ha = 'center', va = 'bottom',
            fontsize=10,
            fontweight="bold",
            horizontalalignment="center",
            verticalalignment="center",
        )
        #
        # Plot orbitals if available.
        #
        from matplotlib.offsetbox import AnnotationBbox, OffsetImage
        try:
            from matplotlib._png import read_png
        except ModuleNotFoundError:
            from matplotlib.image import imread as read_png

        filename = f"mo_{str(indices[i])}.png"
        try:
            offset = 100
            if i % 2 == 0:
                offset = 50
            arr_hand = read_png(filename)
            imagebox = OffsetImage(arr_hand, zoom=zoom)

            ab = AnnotationBbox(
                imagebox,
                (theta[i], r[i] + offset - 20),
                xycoords="data",
                frameon=False,
            )
            ax.add_artist(ab)
            # print also outer indices
            if i % 2 != 0:
                ax.annotate(
                    indices[i],
                    # i+1+orbinit,
                    xy=(theta[i], r[i] + 60),
                    xytext=(0, 0),
                    textcoords="offset points",  # ha = 'center', va = 'bottom',
                    fontsize=10,
                    fontweight="bold",
                    horizontalalignment="center",
                    verticalalignment="center",
                )
        except:
            continue

    ax.yaxis.set_data_interval(0, 22.5)
    ax.xaxis.set_major_formatter(NullFormatter())
    ax.yaxis.set_major_formatter(NullFormatter())
    legend = []

    ax.plot(
        [theta[0], theta[0]],
        [radius[0], radius[0]],
        "",
        lw=2,
        color="w",
        zorder=0,
    )
    for ind in range(len(i12val)):
        ind1_ = np.where(indices == i12ind1[ind] + 1)
        ind2_ = np.where(indices == i12ind2[ind] + 1)
        if i12val[ind] >= 1.0 and i12val[ind] >= thresh:
            color = "black"
            lt = "-"
            lw = 2
            zorder = 4
        elif i12val[ind] >= 0.1 and i12val[ind] >= thresh:
            color = "b"
            lt = "-"
            lw = 2
            zorder = 4
        elif (
            i12val[ind] >= 0.01 and i12val[ind] < 0.1 and i12val[ind] >= thresh
        ):
            color = "r"
            lt = "--"
            lw = 1
            zorder = 3
        elif (
            i12val[ind] >= 0.001
            and i12val[ind] < 0.01
            and i12val[ind] >= thresh
        ):
            color = "#32cd32"
            lt = "-."
            lw = 1
            zorder = 2
        elif (
            i12val[ind] >= 0.0001
            and i12val[ind] < 0.001
            and i12val[ind] >= thresh
        ):
            color = "orange"
            lt = ":"
            lw = 1
            zorder = 1
        else:
            continue

        ax.plot(
            [theta[ind1_], theta[ind2_]],
            [r[ind1_], r[ind2_]],
            lt,
            lw=lw,
            color=color,
            zorder=zorder,
        )
    #
    # Define size of cirlces in mutual information plot
    #
    ind_ = np.array([], dtype=int)
    for el in indices:
        ind_ = np.append(ind_, np.where(s1index == el))
    areavalue = s1value[ind_]
    colorvalue = areavalue
    areavalue *= 200

    #
    # Define own colormap:
    #
    endcolor = "#191970"
    midcolor2 = "#ff0000"
    midcolor1 = "#228b22"
    cmap2 = colors.LinearSegmentedColormap.from_list(
        "own2", [midcolor1, midcolor2, endcolor]
    )
    cm.register_cmap(cmap=cmap2)

    #
    # Plot circles of different size and color
    #
    ax.scatter(
        theta,
        r,
        s=areavalue,
        alpha=0.5,
        c=colorvalue,
        cmap=cmap2,
        vmin=0.001,
        vmax=1,
        norm=colors.LogNorm(),
    )

    blue_line = mlines.Line2D(
        [], [], color="blue", marker="", lw=2, ls="-", label="0.1"
    )
    red_line = mlines.Line2D(
        [], [], color="red", marker="", lw=1, ls="--", label="0.01"
    )
    green_line = mlines.Line2D(
        [], [], color="#32cd32", marker="", ls="-.", lw=1, label="0.001"
    )
    orange_line = mlines.Line2D(
        [], [], color="orange", marker="", ls=":", lw=1, label="0.0001"
    )

    #
    # Adjust legend to threshold value
    #
    if thresh >= 0.0001 and thresh < 0.001:
        legend.append(blue_line)
        legend.append(red_line)
        legend.append(green_line)
        legend.append(orange_line)
    if thresh >= 0.001 and thresh < 0.01:
        legend.append(blue_line)
        legend.append(red_line)
        legend.append(green_line)
    if thresh >= 0.01 and thresh < 0.1:
        legend.append(blue_line)
        legend.append(red_line)
    if thresh >= 0.1:
        legend.append(blue_line)

    plt.legend(handles=legend, loc="center", fancybox=True, fontsize=10)

    ax.set_ylim(0, 225)

    plt.savefig(f"{filemanager.result_path('i12.pdf')}", format="pdf", dpi=300)


def plots1(orbinit, orbfinal, s1index, s1value, smax=0.71):
    try:
        import matplotlib.pyplot as plt
        import matplotlib.lines as mlines
        from matplotlib.ticker import NullFormatter
    except ImportError:
        if log.do_warning:
            log.warn("Skipping plots because matplotlib was not found.")
        return

    plt.figure(figsize=(10, 5))

    ax2 = plt.subplot(111)
    ax2.axis([orbinit - 1, orbfinal + 1, 0, smax])
    ax2.vlines(s1index, [0], s1value, color="r", linewidth=2, linestyle="-")
    plt.ylabel("single-orbital entropy")
    plt.xlabel("Orbital index")
    plt.plot(s1index, s1value, "ro", markersize=8)

    plt.savefig(f"{filemanager.result_path('s1.pdf')}", format="pdf", dpi=300)


def read_i12_data(orbinit, orbfinal, thresh, fname):
    index1 = np.array([], dtype=int)
    index2 = np.array([], dtype=int)
    value = np.array([])
    with open(fname) as f:
        for line in f:
            words = line.split()
            if (
                int(words[0]) >= orbinit
                and int(words[1]) >= orbinit
                and int(words[0]) <= orbfinal
                and int(words[1]) <= orbfinal
            ):
                index1 = np.append(index1, int(words[0]) - 1)
                index2 = np.append(index2, int(words[1]) - 1)
                value = np.append(value, float(words[2]))
    return index1, index2, value


def read_s1_data(orbinit, orbfinal, filename):
    index = np.array([], dtype=int)
    value = np.array([])
    with open(filename) as f:
        for line in f:
            words = line.split()
            if len(words) != 2:
                raise ArgumentError(
                    f"Expecting 2 fields on each data line in {filename}"
                )
            index = np.append(index, int(words[0]))
            value = np.append(value, float(words[1]))
    if orbfinal:
        newind = np.where(index >= (orbinit))
        index = index[newind]
        value = value[newind]
        newind2 = np.where(index <= orbfinal)
        index = index[newind2]
        value = value[newind2]
    return index, value


def parse_args():
    parser = argparse.ArgumentParser(
        prog="pybest-entanglement.py",
        description="This script makes an orbital entanglement plot. It "
        "assumes that the files s1.dat and i12.dat are present in "
        "the current directory. These two files will be used to "
        "create the figure orbital_entanglement.png.",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"{parser.prog} (PyBEST version) {__version__}",
    )

    parser.add_argument(
        "threshold",
        type=float,
        help="Orbitals with a mutual information below this threshold will not "
        "be connected by a line.",
    )
    parser.add_argument(
        "-i",
        default=1,
        type=int,
        nargs="?",
        help="The first orbital to be used for the plot. [default=%(default)s]",
    )
    parser.add_argument(
        "-z",
        default=0.03,
        type=float,
        nargs="?",
        help="Zoom factor for orbitals. [default=%(default)s]",
    )
    parser.add_argument(
        "-f",
        default=None,
        type=int,
        nargs="?",
        help="The last orbital to be used for the plot (inclusive). "
        "[default=last orbital]",
    )
    parser.add_argument(
        "-t",
        default="all",
        type=str,
        nargs="?",
        help="Type of graph to be plotted. If not specified (None) both graphs \
              will be generated. Possible values are None, i (mutual information), \
              s (single-orbital entropy) [default=%(default)s]",
    )
    parser.add_argument(
        "--smax",
        default=0.71,
        type=float,
        nargs="?",
        help="The maximum of s1 scale. [default=%(default)s]",
    )
    parser.add_argument(
        "--iname",
        default="i12.dat",
        type=str,
        nargs="?",
        help="File name to be read in for Iij. [default=%(default)s]",
    )
    parser.add_argument(
        "--sname",
        default="s1.dat",
        type=str,
        nargs="?",
        help="File name to be read in for S1. [default=%(default)s]",
    )
    parser.add_argument(
        "--order",
        default=None,
        metavar="i",
        type=int,
        nargs="+",
        help="The order of the indices to be shown on each graph. [default=%(default)s]",
    )

    return parser.parse_args()


def main():
    args = parse_args()

    orbinit = args.i
    orbfinal = args.f
    smax = args.smax
    select = args.t
    zoom = args.z
    iname = pathlib.Path(args.iname)
    sname = pathlib.Path(args.sname)
    order = args.order

    # Read s1.dat and store data
    s1index, s1value = read_s1_data(orbinit, orbfinal, sname)

    if orbfinal is None:
        orbfinal = s1index[-1]

    # Define different order for I12 plot
    if order is None:
        order = np.arange(orbinit, orbfinal + 1)
    if len(order) != (orbfinal - orbinit + 1):
        raise ArgumentError("Order does not have proper length.")

    # Plot only selected graphs
    if select not in ["i", "s", "all"]:
        raise ValueError(f"Unknown type {select}")
    if select in ["s", "all"]:
        plots1(orbinit, orbfinal, s1index, s1value, smax)
    if select in ["i", "all"]:
        # Read i12.dat and store data
        i12index1, i12index2, i12value = read_i12_data(
            orbinit, orbfinal, args.threshold, iname
        )

        ploti12(
            i12index1,
            i12index2,
            i12value,
            orbinit,
            orbfinal,
            args.threshold,
            s1index,
            s1value,
            order,
            smax,
            zoom,
        )


if __name__ == "__main__":
    main()
